@@ -7,11 +7,11 @@
 # Created on 2016-06-25
 #
 
-"""An Alfred 3-only version of :class:`~workflow.Workflow`.
+"""An Alfred 3+ version of :class:`~workflow.Workflow`.
 
-:class:`~workflow.Workflow3` supports Alfred 3's new features, such as
+:class:`~workflow.Workflow3` supports new features, such as
 setting :ref:`workflow-variables` and
-:class:`the more advanced modifiers <Modifier>` supported by Alfred 3.
+:class:`the more advanced modifiers <Modifier>` supported by Alfred 3+.
 
 In order for the feedback mechanism to work correctly, it's important
 to create :class:`Item3` and :class:`Modifier` objects via the
@@ -29,7 +29,7 @@ import json
 import os
 import sys
 
-from .workflow import Workflow
+from .workflow import ICON_WARNING, Workflow
 
 
 class Variables(dict):
@@ -50,12 +50,16 @@ class Variables(dict):
     information.
 
     Args:
-        arg (unicode, optional): Main output/``{query}``.
+        arg (unicode or list, optional): Main output/``{query}``.
         **variables: Workflow variables to set.
 
+    In Alfred 4.1+ and Alfred-Workflow 1.40+, ``arg`` may also be a
+    :class:`list` or :class:`tuple`.
 
     Attributes:
-        arg (unicode): Output value (``{query}``).
+        arg (unicode or list): Output value (``{query}``).
+            In Alfred 4.1+ and Alfred-Workflow 1.40+, ``arg`` may also be a
+            :class:`list` or :class:`tuple`.
         config (dict): Configuration for downstream workflow element.
 
     """
@@ -68,7 +72,7 @@ class Variables(dict):
 
     @property
     def obj(self):
-        """Return ``alfredworkflow`` `dict`."""
+        """``alfredworkflow`` :class:`dict`."""
         o = {}
         if self:
             d2 = {}
@@ -92,10 +96,10 @@ class Variables(dict):
 
         """
         if not self and not self.config:
-            if self.arg:
-                return self.arg
-            else:
+            if not self.arg:
                 return u''
+            if isinstance(self.arg, unicode):
+                return self.arg
 
         return json.dumps(self.obj)
 
@@ -250,7 +254,7 @@ class Modifier(object):
 
 
 class Item3(object):
-    """Represents a feedback item for Alfred 3.
+    """Represents a feedback item for Alfred 3+.
 
     Generates Alfred-compliant JSON for a single item.
 
@@ -328,14 +332,17 @@ class Item3(object):
                 :meth:`Workflow.add_item() <workflow.Workflow.add_item>`
                 for valid values.
 
+        In Alfred 4.1+ and Alfred-Workflow 1.40+, ``arg`` may also be a
+        :class:`list` or :class:`tuple`.
+
         Returns:
             Modifier: Configured :class:`Modifier`.
 
         """
         mod = Modifier(key, subtitle, arg, valid, icon, icontype)
 
-        for k in self.variables:
-            mod.setvar(k, self.variables[k])
+        # Add Item variables to Modifier
+        mod.variables.update(self.variables)
 
         self.modifiers[key] = mod
 
@@ -447,10 +454,10 @@ class Item3(object):
 
 
 class Workflow3(Workflow):
-    """Workflow class that generates Alfred 3 feedback.
+    """Workflow class that generates Alfred 3+ feedback.
 
-    ``Workflow3`` is a subclass of :class:`~workflow.Workflow` and
-    most of its methods are documented there.
+    It is a subclass of :class:`~workflow.Workflow` and most of its
+    methods are documented there.
 
     Attributes:
         item_class (class): Class used to generate feedback items.
@@ -476,18 +483,18 @@ class Workflow3(Workflow):
 
     @property
     def _default_cachedir(self):
-        """Alfred 3's default cache directory."""
+        """Alfred 4's default cache directory."""
         return os.path.join(
             os.path.expanduser(
-                '~/Library/Caches/com.runningwithcrayons.Alfred-3/'
+                '~/Library/Caches/com.runningwithcrayons.Alfred/'
                 'Workflow Data/'),
             self.bundleid)
 
     @property
     def _default_datadir(self):
-        """Alfred 3's default data directory."""
+        """Alfred 4's default data directory."""
         return os.path.join(os.path.expanduser(
-            '~/Library/Application Support/Alfred 3/Workflow Data/'),
+            '~/Library/Application Support/Alfred/Workflow Data/'),
             self.bundleid)
 
     @property
@@ -522,9 +529,11 @@ class Workflow3(Workflow):
 
         return self._session_id
 
-    def setvar(self, name, value):
+    def setvar(self, name, value, persist=False):
         """Set a "global" workflow variable.
 
+        .. versionchanged:: 1.33
+
         These variables are always passed to downstream workflow objects.
 
         If you have set :attr:`rerun`, these variables are also passed
@@ -533,9 +542,15 @@ class Workflow3(Workflow):
         Args:
             name (unicode): Name of variable.
             value (unicode): Value of variable.
+            persist (bool, optional): Also save variable to ``info.plist``?
 
         """
         self.variables[name] = value
+        if persist:
+            from .util import set_config
+            set_config(name, value, self.bundleid)
+            self.logger.debug('saved variable %r with value %r to info.plist',
+                              name, value)
 
     def getvar(self, name, default=None):
         """Return value of workflow variable for ``name`` or ``default``.
@@ -560,6 +575,9 @@ class Workflow3(Workflow):
                 turned on for your Script Filter, Alfred (version 3.5 and
                 above) will filter against this field, not ``title``.
 
+        In Alfred 4.1+ and Alfred-Workflow 1.40+, ``arg`` may also be a
+        :class:`list` or :class:`tuple`.
+
         See :meth:`Workflow.add_item() <workflow.Workflow.add_item>` for
         the main documentation and other parameters.
 
@@ -575,6 +593,9 @@ class Workflow3(Workflow):
                                match, valid, uid, icon, icontype, type,
                                largetext, copytext, quicklookurl)
 
+        # Add variables to child item
+        item.variables.update(self.variables)
+
         self._items.append(item)
         return item
 
@@ -678,7 +699,36 @@ class Workflow3(Workflow):
             o['rerun'] = self.rerun
         return o
 
+    def warn_empty(self, title, subtitle=u'', icon=None):
+        """Add a warning to feedback if there are no items.
+
+        .. versionadded:: 1.31
+
+        Add a "warning" item to Alfred feedback if no other items
+        have been added. This is a handy shortcut to prevent Alfred
+        from showing its fallback searches, which is does if no
+        items are returned.
+
+        Args:
+            title (unicode): Title of feedback item.
+            subtitle (unicode, optional): Subtitle of feedback item.
+            icon (str, optional): Icon for feedback item. If not
+                specified, ``ICON_WARNING`` is used.
+
+        Returns:
+            Item3: Newly-created item.
+
+        """
+        if len(self._items):
+            return
+
+        icon = icon or ICON_WARNING
+        return self.add_item(title, subtitle, icon=icon)
+
     def send_feedback(self):
         """Print stored items to console/Alfred as JSON."""
-        json.dump(self.obj, sys.stdout)
+        if self.debugging:
+            json.dump(self.obj, sys.stdout, indent=2, separators=(',', ': '))
+        else:
+            json.dump(self.obj, sys.stdout)
         sys.stdout.flush()
